# SyrusJS

## v1.34.0

requires `apexOS >= 21.40.2`

### Fatigue

- Updated codebase to use new ECU.d structure
- Support for mdsm7 fatigue signals (prefix with `@fatigue_sensor.` or `@mdsm7.fatigue_sensor.`)
  - camera_blocked
  - distraction
  - fatigue_alarm
  - fatigue_warning
  - phone
  - smoking

```syruslang
# either signal will work
define event fatigue_distraction fieldset=defeault ack=seq group=fatigue label=distract trigger=@mdsm7.fatigue_sensor.distraction
define event fatigue_distraction fieldset=defeault ack=seq group=fatigue label=distract trigger=@fatigue_sensor.distraction
```

### Bluetooth

- BT destination point support `define destination bluetooth_d taip bluetooth://_:_ ack=disabled`
- BT call button signals `@bluetooth.button.call_2x` `@bluetooth.button.signal`
- BT destination points retry indefintely until apx-bt returns succesful (and messages are acked where applicable)

#### Signal, variable and value improvements

`set [variable|value] [name] [value]` infers type (true, false, number, string)

- `set variable new_var "lorem"` or `set variable new_var lorem`
  - {{$variables.new_var}} = “lorem”
- `set variable new_var "lorem ipsum"`
  - {{$variables.new_var}} = “lorem ipsum”
- `set value $gnss.latitude == 25.123456`
- works for any non internal signal (non @ signals)

`fire signal signal_name` does not require @ as prefix

#### TAIP Fixes

- SSL and QXAEC:: do not requiere an ;ID or ;SI when sent remotely to the device
  - >SSLget events<
  - >QXAEC::apx-gps state<
- fixed QSSIP response
- IB/IS syrus3 compatability fix
- AW tag sends empty instead of *

### Other

- http destinations respect url schema
- long commands can be split into multiple lines
- actions can have multiple commands (requires indentation)

### Multiline support

long commands can now be split, indentation is not required but recommended for legibility

```syruslang

define counters globals speed_threshold=100kph
  rpm_threshold=3000rpm
  begin_idle_time=300sec
  odometer=0mts
  ignition_time=0mts

define fieldset default
  fields=$io,$gnss,$net,$ecu,
  VO:$counters.globals.odometer,CE:$counters.globals.ignition_time,
  CL:$counters.globals.idle_time,CS:$counters.globals.over_speed,
  CR:$counters.globals.over_rpm
```

actions can execute multiple commands, any indented commands following `define action` will be assumed to be part of the action.

```syruslang

define action action1 trigger=signal
  set variable test1 1
  set variable test2 2
  set variable test3 3
```

## v1.32.10

- Improved ack support for TAIP/tcp, supported modes are `imei`(default), `disabled` and `seq`
- ECU Errors, SPI, FMI, OC, SM and MIL support
- ECU events supports signals, see the respective pgn in the SDK ECU.d
- MOVON and Mobileye support, loaded from internal json files.

### Bugfixes

- Queues are loaded in proper order
- Improved callbacks for TCP

## v1.31.7

### Fatigue Sensor support

Signals

- @fatigue_sensor.fatigue_remind
- @fatigue_sensor.fatigue_warning
- @fatigue_sensor.fatigue_alarm
- @fatigue_sensor.distraction
- @fatigue_sensor.no_portrait
- @fatigue_sensor.photo

```syruslang
define event fatigue_remind fieldset=defeault ack=seq group=fatigue photo=fatigue_sensor.fatigue_remind label=remind trigger=@fatigue_sensor.fatigue_remind
define event fatigue_warning fieldset=defeault ack=seq group=fatigue photo=fatigue_sensor.fatigue_warning label=warning trigger=@fatigue_sensor.fatigue_warning
define event fatigue_alarm fieldset=defeault ack=seq group=fatigue photo=fatigue_sensor.fatigue_alarm label=alarm trigger=@fatigue_sensor.fatigue_alarm
define event fatigue_distraction fieldset=defeault ack=seq group=fatigue photo=fatigue_sensor.distraction label=distract trigger=@fatigue_sensor.distraction
define event fatigue_no_driver fieldset=defeault ack=seq group=fatigue photo=fatigue_sensor.no_portrait label=no_driver trigger=@fatigue_sensor.no_portrait
define event fatigue_photo_captured fieldset=defeault ack=seq group=fatigue photo=fatigue_sensor.photo label=photo trigger=@fatigue_sensor.photo
```

### MDT support

- @mdt.message signal is required on the trigger to use the message, it's recommended that you do not mix this with other signals

```syruslang
define event mdt group=mdt fieldset=default code=0 trigger=@mdt.message
```

- TAIP: `STX` is now reserved for MDT, use `SSL` for sending syruslang commands

### Bugfixes

- false|true|Number are parsed as their respective types, not as strings (eg repeated=false)
- min_duration improvements
- improved signal propagation

## v1.30.1

### TPMS support

Signals supported

- @tpms.alarm_ok
- @tpms.sensor_mute
- @tpms.sensor_defective
- @tpms.tire_leak
- @tpms.temperature_warning
- @tpms.over_inflation_critical
- @tpms.over_inflation_warning
- @tpms.under_inflation_warning
- @tpms.under_inflation_critical

## v1.29.0

- Satcom destination support
- TAIP QXASMS, II support (fieldset $satcom)
- improved ack support for TAIP protocol (seq, disabled, imei)
- general tweaks and refactors

### Timers support

```syruslang
define timer timer_name duration=10sec [delay=10sec] [repeat=true] [enabled=true]
# enable|start timer (if not enabled in definition)
start timer timer_name
# stop|disable timer
stop timer timer_name
# use the timer signal in your actions
define action countdown_complete trigger=@timers.timer_name set out2 on
# enable/disable timer from other actions
define action timer_cancel trigger=... stop timer timer_name
```

### IButton

```syruslang
add ibutton john_smith DA00001C3C9C3101
define fieldset taip fields=IB:$ibutton.connected.id,IS:$ibutton.last.id
define fieldset json fields=$ibutton

## SIGNALS
# authorized driver
define signal auth_ibutton $ibutton.connected.whitelisted == true

# unauthorized driver
define signal unauth_ibutton $ibutton.connected.whitelisted == false
# define signal ibutton $ibutton.connected

## EVENTS

# authorized driver
define event authIbutton group=tracking fieldset=taip ack=seq label=ibmtch code=61 trigger=auth_ibutton
# unauthorized driver
define event unauthIbutton group=tracking fieldset=taip ack=seq label=ibnodet code=62 trigger=unauth_ibutton

## ACTIONS
# action to deactivate output2 if driver authorized and speak welcome message
define action auth_driver_deactivate_out trigger=auth_ibutton set out2 off
define action speakActionIbutton trigger=auth_ibutton speak lang=en 'Welcome {{$ibutton.authorized.last.alias}}. Syrus wishes you an excellent trip'
# action to activate output2 if driver is unauthorized
define action unauth_activate_out trigger=unauth_ibutton set out2 on
define action speakActionNoIbutton trigger=unauth_ibutton speak lang=en 'You are not an authorized driver. Please contact an administrator'
```

### Temperature

```syruslang
define group temp
add temperature_sensor temp1 8D01144D07DBAA28
add temperature_sensor temp2 8D01144D07DBAB28
add temperature_sensor temp3 8D01144D07DBAC28

## FIELDSETS
define fieldset taip fields=EA_A:$temp.aliases.temp1.value,EA_B:$temp.aliases.temp2.value,EA_C:$temp.aliases.temp3.value
define fieldset json fields=$temp

## SIGNALS
# temp1 connected
define signal temp1_connected $temp.aliases.temp1.connected
define signal temp2_connected $temp.aliases.temp2.connected
define signal temp3_connected $temp.aliases.temp3.connected
# temperature above 5°C
define signal temp1_above $temp.aliases.temp1.value > 5
define signal temp2_above $temp.aliases.temp2.value > 5
define signal temp3_above $temp.aliases.temp3.value > 5
# temperature below -8°C
define signal temp1_below $temp.aliases.temp1.value < 8
define signal temp2_below $temp.aliases.temp2.value < 8
define signal temp3_below $temp.aliases.temp3.value < 8

## EVENTS
# temp1 Connection
define event temp1Connected group=temp fieldset=taip ack=seq label=temp1det trigger=temp1_connected
define event temp1Disconnected group=temp fieldset=taip ack=seq label=temp1nodet trigger=temp1_connected,not
# temp above
define event tempAbove group=temp fieldset=taip ack=seq label=tempabv code=61 trigger=temp1_above,temp2_above,or,temp3_above,or
# temp below
define event tempBelow group=temp fieldset=taip ack=seq label=tempblw code=62 trigger=temp1_below,temp2_below,or,temp3_below,or
```

### bugfixes

- application/instance name issue
- destination enable/disable

## v1.26.0

- Usage of new queue algorithm that reduce memory footprint and increase maximum stack of events to almost the whole available memory
- fixed value of internal signal $timestamp
- added new signal equivalent to $timestamp for spanish language \$tiempo
example

```syruslang
set value $tiempo # it returns 'Viernes, Diciembre 4, 2020 8:17 PM'
```

## v1.25.0

- added json in ecu for multiple key values for fieldset=$ecu:$ecu, o fieldset=$ecu,
- added new dev commands for get signals, get variables, get actions, get events, get fieldsets, get tracking_resolutions, get variables
get the raw definition as syruslang from configuration separated by `;`
example:
get signals # returns 'define signal out2on $io.out2;define signal out1on $io.out1'
- fixed issue that without a tracking_resolution syrusjs is not updating internal $gnss references, now data is available every second

## v1.24.0

- added mqtt support for syruslang command in subscribe using new param commands_sub
- added mqtt support for syruslang command in publish using new param commands_pub
- fixed json type of key are being always string under certain conditions

## v1.23.0

- added inputs 3,4,5,6 and 7 and ouputs 3 and 4 support for TAIP
- added short circuit detection to outputs support for TAIP
- added support for changing output 3 and output 4 using TAIP serial expander commands

## v1.22.1

- added 2 new internal signal @geofence.enter and @geofence.exit fired in every geofence entered or exited

```syruslang
# geofence.enter and @geofence.exit are internal signal so no need to define a new signal
define event geoIn code=34 label=geoin fieldset=default trigger=@geofence.enter
define event geoOut code=35 label=geoout fieldset=default trigger=@geofence.exit
```

example:

- added 2 new internal signal @geofence.group.enter and @geofence.group.exit fired in every geofence's group entered or exited
- DEV: added new command fire signal to manually trigger signals.

Example:

```syruslang
fire signal IN1ON #(only available from remote console).
```

- DEV: added new command set value to manually setting non internal values.
Example

```syruslang
set $geofences.test.inside true  #(only available from remote console).
```

## v1.22.0

- added compatibility with apex-20.46.2 and new redis instance
- added new flat retain in event for mqtt retain feature
example:
define event trackpoint retain=true code=1 label=trckpnt fieldset=default trigger=@tracking_resolution.mqtt.signal # this event will be retain by the mqtt broker as the last update available

## v1.21.0-beta

- fix inputs/outputs are on true after reset
- added support for configuration.json using json schema
- now it's possible change the configuration file using the json schema included on the app.
- now it's possible change the destinations file using the json schema included on the app.

## v1.20.0-beta

- fix issue double space on destinations
- upgrade syrus4-nodejs to support future versions of ApexOS (>20.44)
- speak alias now support multiple languages speak lang=es,speak lang=en, speak lang=fr, speak without arguments is passed as lang=en
- fixed ecu for TAIP commands
- since apex-20.44.00 syrusjs will stop processing geofences within geofences.syrus.conf file, you should consider import an export geofences from UI and apx-geofences-manager integrated tool, since is not blocking for the app and has much more options.
- added new command get detination_state that returns enable/disable for the connection:
  example

```syruslang
get destination_state peg
# must return `enable`
```

- improve support for HTTP/HTTPS destinations now support authentication, headers, params, method (by default POST), an better error logs.
- added simple schema definition for json schema support

## v1.19.1-beta

- fix issue that broken syrusjs when command depending of units are proccesed

## v1.19.0-beta

Starting with this version units are more explicit for arguments. follow specifications for syruslang new valid units are

1. speed=kph,mph,m/s
2. heading=deg,rad
3. dist=km,mts,mile,ft
4. time=hr,min,sec
5. cycle=rpm,hz
   old units schema will be supported but will emit a warning

- added new argument for actions and event called rate. Avoid fires event/action multiple times in x time defined.
  example:

```syruslang
define tracking_resolution 5sec
# avoids this event fires more than 5 times in 1min, units could be sec,min,hr
define event trckpnt code=1 label=trckpnt rate=5/1min trigger=@tracking_resolution.track.signal
```

- added new command 'send event EVENT_NAME' for forcely fire events within the destinations points
- added new command 'trigger action ACTION_NAME' for forcely trigger an event
- added new commands 'enable destination NAME' and 'disable destination NAME'. using this commands destination will be disconnected and stop sending events and receiving commands, but event will queued for connection will be available again.
- support for arguments subscribe and publish in mqtt destination

```syruslang
# DEPECRATION WARNING
# define destination mqtt_broker json mqtt://test.mosquitto.org:1883 protocol="mqtt" channelRx="dev/{{$modem.imei}}/messagestx" channelTx="dev/{{$modem.imei}}/messagestx"
# REPLACE FOR
define destination mqtt_broker json mqtt://test.mosquitto.org:1883 protocol="mqtt" subscribe="dev/{{$modem.imei}}/messagestx" publish="dev/{{$modem.imei}}/messagestx"
```

## v1.18.0-beta

- support SXAVO command in TAIP
- implement allowed_interface, allowed_ip and allowed_ssid (, for wifi only) to destinations, so destinations can be set to only works in specific enviroments (interface,ip,ssid)
  example:

```syruslang
# this connection will automatic disable outside wifi network and buffer the commands, when wifi is connected again all the messages on the buffer will be sent
define destination mqtt_broker json mqtt://test.mosquitto.org:1883 protocol="mqtt" subscribe="dev/messagestx" publish="dev/messagestx" allowed_interface=wifi
```

- allow use curly brackets to destination config and other commands
  example:

```syruslang
define destination mqtt_broker json mqtt://test.mosquitto.org:1883 protocol="mqtt" subscribe="dev/{{$modem.imei}}/messagestx" publish="dev/{{$modem.imei}}/messagestx"
```

- Implement variables
  example:

```syruslang
define variable my_var "hello"
# you can define actions for change variables or just use as constants
set variable my_var "good bye"
define action aloha trigger=ignSwitch speak '{{$variables.my_var}}, driver'
```

- added new syruslang signal variable $mph, $kph, $speed, $timestamp, those are version more humanized of the $gnss.mph, $gnss.kph, $gnss.speed and $gnss.timestamp
  example:

```syruslang
speak 'today is {{$timestamp}}. The current speed is {{$kph}} kilometers per hour'
```

- supervise configuration.syruslang for changes and restart automatically the app

## v1.17.4-beta

- GPS data update every second, improve signal change detecttion
- added new alias bluetooth for apx-bt commands

## v1.17.3-beta

- Replace underscore (\_) for blank spaces in speak command

## v1.17.2-beta

- fix is inside notification on geofences and groups

## v1.17.1-beta

- added new configuration file geofences.syrus.conf. geofences.syrus.conf is only evaluated when changes are detected; Avoid to add commands different than signals on this file. (recommended move all geofences previously on configuration.syrus.conf to this file.)
- added new command speak alias for 'apx-mdm tts'. Usage: speak 'text to speech test'. only support english, the text must be encapsulated over single quoutes `'`
- added new values @last_geofence.name,@last_geofence.state and @last_geofence.time contains information about last geofence reported for apx-geofences core tool
- added ability to concatenate syrusjs signal variable on some exec commands.
  example:

```syruslang
$TXspeak 'current location.  Latitude: {{$gnss.latitude}}.  Longitude:{{$gnss.longitude}}. Last Geofence reported: {{@last_geofence.name}} {{@last_geofence.state}}'
```

Variable must be inside notation {{ }}, variable is replaced in real time with current value of the syrusLang signal

## v1.16.1-beta

- fix ecu odometer parameter for TAIP
- fix ecu battery parameter for TAIP

## v1.16.0-beta

- geofences are now evaluated using apx-geofences utils from apexOS core
- added support for detecting process exit and clean disconnection from all the destinations point
- added new 38 parameters from ecu
- added new command set @ecu_params_response
- ecu signals and values now are accesible using id instead of aliases name
  example:

```syruslang
# ENGINE TOTAL FUEL USED
# previously $ecu.eng_tot_fuel now is
$ecu.fee9_5-8
```

## v1.15.2-beta

- added counters
- added new command define counters

```syruslang
# only name is mandatory
define counters name speed_threshold=50kph rpm_threshold=3000rpm begin_idle_time=300s odometer=0mts ignition_time=0s idle_time=0s over_speed=0s over_rpm=0 hard_brakes=0 harsh_fwd_acceleration=0
```

- added new command

```syruslang
# only name is mandatory
# require at least one new value to set
set counters name speed_threshold=50kph rpm_threshold=3000rpm begin_idle_time=300s odometer=0mts ignition_time=0s idle_time=0s over_speed=0s over_rpm=0 hard_brakes=0 harsh_fwd_acceleration=0
```

- added new command

```syruslang
stop counters name
```

- added new command

```syruslang
start counters name
```

- added new command

```syruslang
delete counters name
```

- added new values to syruslang signals

```syruslang
# $counters.namespace.odometer
# $counters.namespace.ignition_time
# $counters.namespace.idle_time
# $counters.namespace.over_speed
# $counters.namespace.over_rpm

# example signal
define signal oil_change_required $counters.tecno.odometer > 20000
```

- added support to syruslang fieldset to use counters

```syruslang
# Example

define counters globals speed_threshold=50kph rpm_threshold=3000rpm begin_idle_time=300s odometer=0mts ignition_time=0s idle_time=0s over_speed=0s over_rpm=0 hard_brakes=0 harsh_fwd_acceleration=0

define counters driver speed_threshold=50kph rpm_threshold=3000rpm begin_idle_time=300s odometer=0mts ignition_time=0s idle_time=0s over_speed=0s over_rpm=0 hard_brakes=0 harsh_fwd_acceleration=0

define fieldset default fields=$io,$ecu,VO:$counters.globals.odometer,CE:$counters.globals.ignition_time,CL:$counters.globals.idle_time,CS:$counters.globals.over_speed,CR:$counters.globals.over_rpm,CV00:$counters.driver.odometer,CV01:$counters.driver.ignition_time


define signal driver_connected $net_wifi.ssid == "MOBILE HOTSPOT"
define action trigger=driver_connected start counters driver
define action trigger=driver_connected,not stop counters driver
```

- changed voltage thresholds and 100% to 4000 mV

## v1.14.4-beta

- improve tracking resolution detection arguments
- fix case when only tracking resolution had time with `m`
- added syruslang syruslang conversion for radianes

## v1.14.3-beta

- fixed empty arguments for tracking resolutions

## v1.14.2-beta

- fixed heading on taip events
- group warning when no group detected

## v1.14.0-beta

- changed signals from @tracking_resolution.signal to @tracking_resolution.{namespace}.signal where namespace is the name of the tracking resolution on define tracking_resolution command
- set tracking_resolution command changed to support new configuration of the tracking resolution, ej:

```syruslang
define signal ignOnSignal $io.ign == true
define tracking_resolution pegasus 10m
define action ignOn trigger=ignOnSignal set @tracking_resolution pegasus 120s 25deg 100mts
```

- removed usage of set @tracking_resolution {name} as usage for change active tracking resolution in favor to have multiple tracking resolutions at the same time
