import axios from "axios"

import { Channel } from "./Channel.js"

export class HTTP extends Channel {
	constructor(config) {
		super(config);
		this.onConnect = null;
		this.onDisconnect = null;
		this.onMessage = null;
		this.name = config.name;
		this.protocol = config.protocol;
		this.url = config.url;
		this.host = config.host;
		this.port = config.port;
		this.method = config.method || "POST";
		this.params = config.params;
		this.username = config.username;
		this.password = config.password;
		this.headers = config.headers;
		this.timeout = config.timeout;
	}

	destroy() {
		return this.promise;
	}

	async connect() {
		await this.ready;
		if (this.onConnect) this.onConnect();
		this.sendBuffer();
	}

	disconnect() {
		if (this.onDisconnect) this.onDisconnect();
	}

	send(data) {
		this.queue.push(JSON.stringify(data));
		if (!this.active) return false;
		if (this.queue.length == 1) {
			this.sendBuffer();
		}
	}

	sendBuffer() {
		if (this.queue[0] && this.active) this._send(this.queue[0]);
	}

	async _send(data) {
		let request = {
			method: this.method,
			url: this.url,
			data: data,
			params: this.params,
			headers: this.headers,
			timeout: this.timeout || 5000
		}
		if (this.username && this.password) {
			request.auth = Buffer.from(`${username}:${password}`, "utf8").toString("base64")
		}
		console.log(`http(${this.name}): request`, request)
		try{
			let resp = await axios.request(request)
			console.log(`http(${this.name}): response`, resp.status, resp.data)
			if (this.onMessage) this.onMessage(resp.data)
			this.queue.shift()
			this.sendBuffer()
		} catch (err){
			console.error(
				`http(${this.name}): error`,
				err.response ?
					`code: ${err.response.status}, message: ${err.response.statusText}, message: ${err.response.data}`
					: `${err.code}`
			)
			// client error, discard message
			if (err.response && err.response.status == 400){
				this.queue.shift()
				return
			}
			if (this.onDisconnect) this.onDisconnect(err)
			// all other errors (unauthorized, forbidden, try again
			setTimeout(() => {
				this.sendBuffer()
			}, 60 * 1000);
		}
	}
}

