export type State = "enable" | "disable";

export interface SyrusLangAction {
    readonly command: string, // the action string. example: define signal or get action
    readonly persistent: boolean, // if the command is persistent on the appends file
    readonly raw?: string // the raw command from syruslang script
}

export interface destination extends SyrusLangAction {
    name: string,
    protocol: string,
    url: string,
    port?: string | number, // Not required when its file
    protocolId?: string
    channelRx?: string
    subscribe?: string
    channelTx?: string
    publish?: string
    channel?: string
    headers?: string
    params?: string
}

export interface tracking_resolution extends SyrusLangAction {
    heading?: number
    time?: number
    distance?: number
}

export interface signal extends SyrusLangAction {
    variable: string
    operator: string
    comparator: string
    min_duration?: number
}

export interface group extends SyrusLangAction {
    meta?: string
    fieldset?: string
    destination?: string
}

export interface fieldset extends SyrusLangAction {
    fields: string
}

export interface Event extends SyrusLangAction {
    signals: string, // Postfix signals
    code: string,
    label: string,
    group?: string,
    fieldset?: string,
    ack?: string,
    retain?: string, //Only mqtt is the event has the flag retain
    destinations?: string,
    rate?: [number, number] // number of events, time period in seconds
}

export interface geofence extends SyrusLangAction {
    group?: string
    lngLats: string
    radius: number
}

export interface counters extends SyrusLangAction {
    odometer: number //	initital distance traveled with ignition ON (meters)
    ignition_time: number // initital ignition time (seconds)
    idle_time: number // initital time spent in idling (seconds)
    over_speed: number // initital time spent over speed (seconds)
    over_rpm: number //	initital time spent over RPMs (seconds)
    hard_brakes: number // initital amount of hard brake events (count)
    harsh_fwd_acceleration: number // initital amount of harsh forward acceleration events (count)
    forceSet: boolean, // Force to set counter values to initial values  if the counters already exists
    speed_threshold: number	// 50kph	The over_speed counter will increase for every second the device is above this speed
    rpm_threshold: number	// 3000rpm	The over_rpm counter will increase for every second the device is above this rpm
    begin_idle_time: number	// 300sec	The idle_time counter begins to increase after (no movement detected AND ignition ON) for longer than the begin_idle_time
}

export interface namespace extends SyrusLangAction { }

export interface GeneralCommand extends SyrusLangAction {
    io?: string
    type?: string
    state?: State,
    signal_name?: string,
    returnValue?: boolean,
    value?: any,
}

export interface SyrusLangJsonAction extends destination, tracking_resolution, signal, group, fieldset, Event, geofence, counters, namespace, GeneralCommand { }
