import { Utils } from "syrus4-nodejs"

import dayjs from "../../utils/dayjs.js"

export const BacklogMixin = Base => class extends Base {
	async processBacklog(file, label, code, attempt=1) {
		if (!file) return
		attempt++
		if (attempt == 6) {
			console.error(`${file} was never resolved`)
			return
		}
		const backlog = await Utils.OSExecute(`apx-gps backlog status --name=${file}`)
		console.log(backlog)
		if (backlog.state != "READY") {
			await Utils.$sleep(10 * 1000)
			return await this.processBacklog(file, label, code, attempt)
		}
		const data = await Utils.OSExecute(`cat ${backlog.path}`)
		const lines = data.split('\n')
		const _headers = lines.shift()
		let seq = 0
		for(const line of lines){
			if (!line.trim().length) continue
			const [time, fix, qlty, hdop, pdop, vdop, satused, dec, lat, lon, alt, speed, track] = line.split(',')
			const event = {
				'$$backlog': true,
				'$gnss.timestamp': dayjs(time).unix(),
				'$gnss.fix': fix,
				'$gnss.qlty': qlty,
				'$gnss.hdop': hdop,
				'$gnss.pdop': pdop,
				'$gnss.vdop': vdop,
				'$gnss.satsActive': satused,
				'$gnss.dec': dec,
				'$gnss.latitude': lat,
				'$gnss.longitude': lon,
				'$gnss.altitude': alt,
				'$gnss.speed': Number(speed) * 0.277778,
				'$gnss.bearing': track,
			}
			const taip = await this.JSONtoTAIP(event, {}, {}, label, code);
			const message = `>RXAKL${seq.toString().padStart(4, 0)}:${taip.slice(1)}`
			seq++
			// console.log(message)
			this.push(message)
		}
		return
	}
}
