
export function pad(number, digits, is_decimal = false) {
	if (String(number).length > digits) {
		number = String(number).substring(0, digits);
	}
	if (is_decimal) {
		return String(number) + Array(Math.max(digits - String(number).length + 1, 0)).join("0");
	}
	return Array(Math.max(digits - String(number).length + 1, 0)).join("0") + number;
}

/**
 * Set Latitude data in  Taip/Syrus format data
 *
 * @param {number} lat
 * @returns Latitude string parser
 *
 * @memberOf Parser
 */
export function getLat(lat = 0.0) {
	let integer = Math.floor(Math.abs(lat));
	let symbol = lat > 0 ? "+" : "-";
	let decimal = (lat + "").split(".")[1];
	if (!decimal) decimal = "0";
	return `${symbol}${pad(integer, 2)}${pad(decimal, 5, true)}`;
}

/**
 * Set longitude data in  Taip/Syrus format data
 *
 * @param {number} long
 * @returns longitude string parser
 *
 * @memberOf Parser
 */
export function getLon(long = 0.0) {
	let integer = Math.floor(Math.abs(long));
	let symbol = long > 0 ? "+" : "-";
	let decimal = (long + "").split(".")[1];
	if (!decimal) decimal = "0";
	return `${symbol}${pad(integer, 3)}${pad(decimal, 5, true)}`;
}

export async function getSC(fields, data) {
	try {
		if (fields['$modem']) {
			let $modem = {
				RAT: data['$modem.rat'],
				MCC_MNC: data['$modem.mcc_mnc'],
				LAC: data['$modem.lac'],
				CID: data['$modem.cid'],
				RSSI: data['$modem.rssi'],
			}
			let { RAT, MCC_MNC, LAC, CID, RSSI } = $modem;
			let [MCC, MNC] = [MCC_MNC.slice(0, 3), MCC_MNC.slice(3)];
			CID = parseInt(CID, 16);
			if (isNaN(CID)) CID = '';
			let result = `;SC=${RAT},${MCC},${MNC},${LAC},${CID},${RSSI}`
			result = result.replace(/(undefined|null)/g, '')
			if (result == ';SC=,,,,') return ""
			return result
		}
		return ""
	} catch (e) {
		console.error(e)
		return ""
	}
}

export function __noop() { }
