import fs from "fs"
import { Apps } from "syrus4-nodejs"
import { app_data_folder, configuration_file, syruslangParser } from "./globals.js"
// Default Configuration
const default_config = {
	namespace: "syrusjs",
	configuration_file: "configuration.syrus.conf",
	destination_file: "destinations.syrus.conf",
	appends_file: "appends.syrus.conf"
};

/**
 *	this commands receive a configuration from where are syruslang script and returns syruslang command actions in array
 * @param {Object} configuration object from getConfiguration with define destinationsPath, configPath and appendsPath
 *  @returns {Array} syruslang commands array
 */
export function getCommandsFromconfigurationFiles(configuration) {
	var { destinationsPath, configPath, appendsPath } = configuration;

	// SyrusLang commands from destination file
	var destination_commands = syruslangParser.process(fs.readFileSync(destinationsPath).toString());
	for (const command of destination_commands) {
		if (command.command != "define destination") {
			console.warn(`${command.raw} is not a destination definiton, and should be removed from this file`);
		}
	}

	// SyrusLang commands from configuration file
	var configuration_commands = syruslangParser.process(fs.readFileSync(configPath).toString());
	for (const command of configuration_commands) {
		if (command.command == "define destination") {
			console.warn(`${command.raw} is a destination definiton, and should be added to the destinations file`);
		}
	}

	// SyrusLang commands from appends file
	var appended_commands = syruslangParser.process(fs.readFileSync(appendsPath).toString());

	return [
		...destination_commands,
		...configuration_commands,
		...appended_commands
	];
}

/**
 *  Function that managed to get all the valid configurations from the different files of config like *.syrus.conf and CONFIGURATION_FILE
 *
 */
export async function getConfiguration() {
	// Get the config from /data/app_data/{instance}/CONFIGURATION_FILE
	// defined by JSON schema and the UI of applications
	var config = await Apps.getConfiguration();
	// merged with the default config
	var configuration = Object.assign({}, default_config, config);
	if (!configuration.namespace) configuration.namespace = "syrusjs";
	configuration.destinationsPath = `${app_data_folder}/${configuration.destination_file}`;
	configuration.configPath = `${app_data_folder}/${configuration.configuration_file}`;
	configuration.appendsPath = `${app_data_folder}/${configuration.appends_file}`;
	configuration.schemaPath = `${app_data_folder}/${configuration_file}`;

	// If no config file app must no run
	if (!fs.existsSync(configuration.configPath)) {
		console.error(`Missing configuration file at: '${configuration.configPath}' `);
		process.exit();
	}
	// If no destination file app must no run
	if (!fs.existsSync(configuration.destinationsPath)) {
		console.error(`Missing destinations file at: '${configuration.destinationsPath}' `);
		process.exit();
	}
	// If no appends file app will create a new one
	if (!fs.existsSync(configuration.appendsPath)) {
		fs.writeFileSync(configuration.appendsPath, "");
	}
	return configuration;
}
