import { Counters as S4Counters } from "syrus4-nodejs"
import { syrusjs_variables, counters } from "./globals.js"

/**
 * create a new S4 Counters and stored in global counters
 * @param {*} config counters parameters
 */
export async function createCounters(config) {
	await S4Counters.startCounters(config);
	counters[config.name] = S4Counters.watchCounters(
		config.name,
		values => {
			onCountersChange(values, config.name);
		},
		console.error
	);
}

/**
 * start/play the counters
 * @param {*} name name of the counters
 */
export function startCounters(name) {
	S4Counters.startCounters({ name: name });
}

/**
 * stop/pause the counters
 * @param {*} name name of the counters
 */
export function stopCounters(name) {
	S4Counters.stopCounters({ name: name });
}

/**
 * callback to execute when a counter is updated, its update the syrusjs variables
 * @param {*} counter the new data
 * @param {*} name name of the counters
 */
function onCountersChange(counter, name) {
	for (const key in counter) {
		syrusjs_variables[`$counters.${name}.${key.toLowerCase()}`] = counter[key];
	}
}

/**
 * Totally delete the counters from the name passed as argument
 * @param {*} name name of the counters
 */
export function deleteCounters(name) {
  if (counters[name]) counters[name].unsubscribe();
  S4Counters.deleteCounters(name)
  delete counters[name];
}
