import { syrusjs_variables, structure, timers } from "./globals.js"

/**
 * creates a new timer
 * @param {*} config counters parameters
 */
export function createTimer(config) {
	console.log(`timers(creating) name=${config.name}`)
	structure.timers[config.name] = config
	timers[config.name] = {
		timeout: null,
		started_at: null
	}
	if(config.enabled) startTimer(config.name)
}

/**
 * restart timer
 * @param {*} name name of the counters
 */
export function startTimer(name) {
	if (!timers[name]) {
		console.warn(`timers: timer ${name} does not exist`)
		return
	}
	let config = structure.timers[name]
	if (!config.duration) return
	if (timers[name].timeout) cancelTimer(name)
	console.log(`timers(starting) name=${name}`)
	timers[name].started_at = Date.now() / 1e3
	timers[name].timeout = setTimeout(onTimerComplete, config.duration, config.name)
}

/**
 * cancel timer
 * @param {*} name name of the counters
 */
export function cancelTimer(name) {
	if (!timers[name]){
		console.warn(`timers: timer ${name} does not exist`)
		return
	}
	if (!timers[name].timeout) return
	console.log(`timers(stop) name=${name}`)
	clearTimeout(timers[name].timeout)
	timers[name].started_at = null
	timers[name].timeout = null
}

/**
 * Totally delete the counters from the name passed as argument
 * @param {*} name name of the counters
 */
export function deleteTimer(name) {
	if (!timers[name]) {
		console.warn(`timers: timer ${name} does not exist`)
		return
	}
	cancelTimer(name)
	console.log(`timers(deleted) name=${name}`)
	delete timers[name];
}

/**
 * callback to execute when a timer is completed, updates syrusjs_variables
 * @param {*} name of the counter
 */
function onTimerComplete(name) {
	console.log(`timers(completed) name=${name}`)
	let config = structure.timers[name]
	let path = `@timers.${name}`
	if (!structure.signals[path]) {
		structure.signals[path] = {
			name: path,
			variable: path,
			operator: null,
			comparator: null,
			min_duration: 0,
			limit: 0,
			active: true
		};
	}
	syrusjs_variables[path] = true;
	setTimeout(() => {
		syrusjs_variables[path] = false;
		timers[name].started_at = null
		timers[name].timeout = null
		if (!config.repeat) return
		console.log(`timers(repeat) name=${name} delay=${config.delay}`)
		setTimeout(startTimer, config.delay, name)
	}, 20);
}
