import net from "net"
import fs from "fs"
import path from "path"

import { app_data_folder } from "../scripts/globals.js"

export function socket_connection(processCommand) {
	return new Promise((resolve, reject)=>{
		let app_data_folder_copy = app_data_folder
		if(app_data_folder_copy.startsWith("/")) app_data_folder_copy = app_data_folder_copy.slice(1)
		const socket_name = path.join("/tmp", app_data_folder_copy.replace(/\//g, "-") + ".sock");
		if(fs.existsSync(socket_name)) fs.unlinkSync(socket_name)
		let unixServerTimeout
		try{
			// This server listens on a Unix socket at within socket_name
			console.log("unix-socket: initializing", socket_name)
			const unixServer = net.createServer();
			unixServerTimeout = setTimeout(() => {
				try{
					console.error("unix-socket: closing due to timeout")
					unixServer.close(console.log)
				} catch(error){
					console.error(error)
				}
				reject("unix-socket: failed to establish")
			}, 10000)
			unixServer.on("connection", (client) => {
				console.log("unix-socket: new connection");
				client.on("data", async data => {
					if(data.toString().length > 1){
						console.log("unix-socket: received command", data.toString());
						let response = await processCommand({ command: data.toString(), isSyrusLang: true });
						client.write((response || "OK") + "\n");
					}
				});
			});
			unixServer.on('listening', ()=>{
				clearTimeout(unixServerTimeout)
				console.log("unix-socket: initialized")
				resolve()
			})
			unixServer.on('error', (error)=>{
				console.error("unix-socket: error:", error)
			})
			unixServer.listen(socket_name);
		} catch(error){
			clearTimeout(unixServerTimeout)
			console.error("unix-socket: failed to establish")
			reject(error)
		}
	})
};
