import { redisClient } from "syrus4-nodejs/dist/Redis.js"
// Prefix added to all the redis keys,to avoid duplicates from different instances
const prepend = process.env.APP_DATA_FOLDER || "."

/**
 *  Intended for use redis as storage wrapper in defining function
 *  could be extended in future to use diffenrent storage than redis
 */
export default {
	__client: redisClient,
	__prepend: prepend,
	async set(key, value) {
		return await redisClient.set(`${prepend}_${key}`, value);
	},
	async get(key) {
		return await redisClient.get(`${prepend}_${key}`);
	},
	async remove(key) {
		return await redisClient.del(`${prepend}_${key}`);
	},
	async getList(key) {
		return await redisClient.lrange(`${prepend}_${key}`, 0, -1);
	},
	async push(key, item) {
		return await redisClient.lpush(`${prepend}_${key}`, item);
	},
	async unshift(key, item) {
		return await redisClient.rpush(`${prepend}_${key}`, item);
	},
	async pop(key) {
		return await redisClient.rpop(`${prepend}_${key}`);
	},
	async shift(key) {
		return await redisClient.rpop(`${prepend}_${key}`);
	},
	async lset(key, index, value) {
		return await redisClient.lset(`${prepend}_${key}`, index, value);
	},
	async slice(key, start = 0, stop) {
		return await redisClient.lrange(`${prepend}_${key}`, start, stop);
	},
	async trim(key, length = 500) {
		return await redisClient.ltrim(`${prepend}_${key}`, 0, length);
	},
	async hset(hash, key, value) {
		return redisClient.hset(`${prepend}_${hash}`, key, value);
	},
	async hget(hash, key) {
		return await redisClient.hget(`${prepend}_${hash}`, key);
	},
	async hgetall(hash) {
		return await redisClient.hgetall(`${prepend}_${hash}`);
	},
	async hdel(hash, key, value) {
		return redisClient.hdel(`${prepend}_${hash}`, key, value);
	}
};
